package com.shanpin.sales.wap.module.base;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.esms.common.entity.GsmsResponse;
import com.shanpin.core.constant.EnvConf;
import com.shanpin.core.http.HttpHelper;
import com.shanpin.core.http.MJsonView;
import com.shanpin.core.status.BaseStatus;
import com.shanpin.core.utils.NumberUtil;
import com.shanpin.sales.wap.module.base.model.SmsTime;

@Controller
@RequestMapping("/wap/sms")
public class SMSController {
	
	private Logger log = LoggerFactory.getLogger(SMSController.class);

	@Autowired
	private SMSService smsService;
	@Autowired
	private CacheService cacheService;
	
	@RequestMapping("/short/code")
	public Object verifyCode(String phone, HttpServletRequest req) {
		if (StringUtils.isBlank(phone)) {
			return MJsonView.resp(BaseStatus.PARAM_INVALID, "空的手机号码");			
		}
		if (!NumberUtil.verifyPhone(phone)) {
			return MJsonView.resp(BaseStatus.PARAM_INVALID, "手机号码不合规范");
		}
		String ip = HttpHelper.getIp(req);
		int count = smsService.add(phone);
		int ip_count = smsService.add(ip);
		if (count > 5 || ip_count > 10) {
			log.warn("领工资限制30分钟内发5条。phone:{} {}  ip:{} {} ", phone, count, ip, ip_count);
			return MJsonView.fail("账号锁定，请稍候再试");
		}
		String num = NumberUtil.genRandomNums(4);
		while (num.startsWith("0")) {
			num = NumberUtil.genRandomNums(4);
		}

		SmsTime st = cacheService.getSMS(phone);
		if (null == st || st.isAllow()) {
			try {
				log.warn("phone:{}  code:{}  ip:{}", phone, num, HttpHelper.getIp(req));
				if(EnvConf.isProd()) {
					GsmsResponse gresp = smsService.send(phone, 
							"验证码："+num+"，5分钟内有效。为了保证安全，请勿向他人泄露。谢谢您使用'领工资'。");
					if (null != gresp && gresp.getResult() == 0) {
						cacheService.putSMS(phone, SmsTime.valueOf(num));
						return MJsonView.ok();
					} else {
						log.error(" short-code result:{}  msg:{}", gresp.getResult(), gresp.getMessage());
						return MJsonView.fail("服务繁忙，请稍候再试");
					}
				} else {
					cacheService.putSMS(phone, SmsTime.valueOf(num));
					return MJsonView.ok();
				}
			} catch (Exception e) {
				log.error("verify-code error:{}", e.getMessage());
				return MJsonView.fail("验证码发送出现了问题。");
			}
		}
		return MJsonView.fail("间隔时长未到");
	}
	

}
