package com.shanpin.sales.wap.module.user;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jetty.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import com.shanpin.core.http.MJsonView;
import com.shanpin.core.status.BaseStatus;
import com.shanpin.core.status.UStatus;
import com.shanpin.core.utils.CheckUtils;
import com.shanpin.core.utils.DecimalUtil;
import com.shanpin.core.utils.NumberUtil;
import com.shanpin.sales.wap.module.base.CacheService;
import com.shanpin.sales.wap.module.base.UserLoginController;
import com.shanpin.sales.wap.module.base.model.Result;
import com.shanpin.sales.wap.module.base.model.SmsTime;
import com.shanpin.sales.wap.module.trade.model.Bonus;
import com.shanpin.sales.wap.module.trade.model.Trade;
import com.shanpin.sales.wap.module.user.model.User;
import com.shanpin.sales.wap.module.user.model.UserBind;
import com.shanpin.sales.wap.module.user.model.UserLogin;

@Controller
@RequestMapping("/wap/user")
public class UserController extends UserLoginController{

	@Autowired
	private CacheService cacheService;
	@Autowired
	private UserService userService;
	
	/**
	 * 用户签入
	 */
	@RequestMapping(value="/login", method=RequestMethod.GET)
	public ModelAndView toLogin() {
		ModelAndView mv = new ModelAndView("wap/user/login");
		return mv;
	}
	@RequestMapping(value="/login", method=RequestMethod.POST)
	public ModelAndView login(@ModelAttribute("phone")String phone, String password) {
		ModelAndView mv = new ModelAndView("wap/user/login");
		if(StringUtil.isBlank(phone) || StringUtil.isBlank(password)) {
			mv.addObject(Result.MSG, "账号或密码不能为空");
			return mv;
		}
		UserLogin userLogin = doSignIn(phone, password);
		if(StringUtil.isNotBlank(userLogin.getMsg())) {
			mv.addObject(Result.MSG, userLogin.getMsg());
			return mv;
		}
		if(userLogin.getBind() == UserBind.phone) { //跳转到身份认证
			mv.setViewName("redirect:identify");
			return mv;
		} else if(userLogin.getBind() == UserBind.idCard) { //跳转到用户账户首页
			mv.setViewName("redirect:account");
			return mv;
		} else { //跳转到用户绑定（TIP:正常逻辑不会进到这里）
			mv.setViewName("redirect:bind");
			return mv;
		}
	}
	
	/**
	 * 用户绑定号码
	 */
	@RequestMapping(value="/bind", method=RequestMethod.GET)
	public ModelAndView toBind() {
		ModelAndView mv = new ModelAndView("wap/user/bind");
		return mv;
	}
	
	@RequestMapping(value="/bind", method=RequestMethod.POST)
	public ModelAndView bind(@ModelAttribute("phone")String phone, String password, String verifyCode) {
		ModelAndView mv = new ModelAndView("wap/user/bind");
		UserLogin userLogin = getUserLogin();
		String openId = userLogin.getOpenId();
		if (StringUtils.isBlank(phone) || !NumberUtil.verifyPhone(phone)) {
			mv.addObject(Result.MSG, "手机号码不合规范");
			return mv;
		}
		if(null == password || password.length() != 32) {
			mv.addObject(Result.MSG, "密码不合规范");
			return mv;
		}
		
		SmsTime st = cacheService.getSMS(phone);
		if(null == st || !st.getNum().equals(verifyCode)) {
			mv.addObject(Result.MSG, "验证码不正确");
			return mv;
		}
		User u = userService.getByPhone(phone);
		if (u != null && (u.getBind() != UserBind.no)) {
			mv.addObject(Result.MSG, "手机号已绑定");
			return mv;
		}
		if(StringUtil.isNotBlank(openId)) {
			u = userService.getByOpenId(openId);
			if (u != null) {
				mv.addObject(Result.MSG, "微信号已绑定");
				return mv;
			}
		}
		int uid = userService.bind(phone, password, openId);
		if (uid > 0) { //重新签入
			doSignIn(phone, password);
			mv.setViewName("redirect:identify");
			return mv;
		}
		mv.addObject(Result.MSG, "手机绑定失败。");
		return mv;
	}
	
	/**
	 * 用户认证身份(完成号码绑定后才能完成身份信息绑定)
	 */
	@RequestMapping(value="/identify", method=RequestMethod.GET)
	public ModelAndView toIdentify() {
		ModelAndView mv = new ModelAndView("wap/user/identify");
		UserLogin userLogin = getUserLogin();
		Trade trade = userService.hasTrade(userLogin.getPhone());
		if(trade == null) {
			mv.setViewName("wap/user/identify_fail");
			return mv;
		}
		mv.addObject("company", trade.getCompany());
		return mv;
	}
	@RequestMapping(value="/identify", method=RequestMethod.POST)
	public ModelAndView identify(@ModelAttribute("realName")String realName, @ModelAttribute("idCard")String idCard) {
		ModelAndView mv = new ModelAndView("wap/user/identify");
		UserLogin userLogin = getUserLogin();
		if (StringUtils.isBlank(realName) || StringUtils.isBlank(idCard)) {
			mv.addObject(Result.MSG, "真实姓名和身份证号码均不能为空");
			return mv;
		}
		if(!CheckUtils.checkIdCard(idCard)) {
			mv.addObject(Result.MSG, "身份证格式不正确");
			return mv;
		}
		int uid = userLogin.getUid();
		boolean r = userService.identify(uid, realName, idCard);
		if(r) { //重新签入
			userLogin.setBind(UserBind.idCard);
			mv.setViewName("redirect:account");
			return mv;
		}
		return MJsonView.fail("身份认证失败。");
	}
	
	/**
	 * 找回密码
	 */
	@RequestMapping(value="/findpsw", method=RequestMethod.GET)
	public ModelAndView toFindPassword() {
		ModelAndView mv = new ModelAndView("wap/user/find_psw");
		return mv;
	}
	
	@RequestMapping(value="/findpsw", method=RequestMethod.POST)
	public ModelAndView findPassword(@ModelAttribute("phone")String phone, String password, 
			String verifyCode) {
		ModelAndView mv = new ModelAndView("wap/user/find_psw");
		if (StringUtils.isBlank(phone)) {
			mv.addObject(Result.MSG, "手机号无效");
			return mv;
		}
		if (StringUtils.isBlank(password) || password.length() != 32) {
			mv.addObject(Result.MSG, "密码格式不正确");
			return mv;
		}
		SmsTime st = cacheService.getSMS(phone);
		if(null == st || !st.getNum().equals(verifyCode)) {
			mv.addObject(Result.MSG, "验证码不正确");
			return mv;
		}
		boolean r = userService.findPassword(phone, password);
		if (r) { //重新签入
			UserLogin  userLogin = doSignIn(phone, password);
			if(userLogin.getBind() == UserBind.phone) {
				mv.setViewName("redirect:identify");
				return mv;
			} else if(userLogin.getBind() == UserBind.idCard) {
				mv.setViewName("redirect:account");
				return mv;
			} else {
				mv.setViewName("redirect:bind");
				return mv;
			}
		}
		mv.addObject(Result.MSG, "号码不存在");
		return mv;
	}
	
	/**
	 * 用户账户
	 */
	@RequestMapping(value="/account", method=RequestMethod.GET)
	public ModelAndView account() {
		ModelAndView mv = new ModelAndView("wap/user/account");
		UserLogin userLogin = getUserLogin();
		int uid = userLogin.getUid();
		Bonus bonus = null;
		if(uid > 0) {
			bonus = userService.bonus(uid);
			//补上员工餐厅信息
			Trade trade = userService.hasTrade(userLogin.getPhone());
			bonus.setCompany(trade.getCompany());
		} else {
			bonus = new Bonus();
		}
		mv.addObject("bonus", bonus);
		return mv;
	}
	
	/**
	 * 申请提现
	 * @param key
	 * @param applyFee  申请金额(单位元)
	 * @return
	 */
	@RequestMapping(value="/apply", method=RequestMethod.POST)
	public Object apply(double applyFee) {
		UserLogin userLogin = getUserLogin();
		int uid = userLogin.getUid();
		if (uid < 1) {
			return MJsonView.resp(UStatus.UNLOGIN);
		}
		if(userLogin.getBind() != UserBind.idCard) {
			return MJsonView.resp(UStatus.UNIDENTIFY);
		}
		//检查提现金额是否合法
		if(applyFee < 99.999 || applyFee > 800.001) {
			return MJsonView.resp(BaseStatus.PARAM_INVALID, "提现金额不在允许的范围");
		}
		applyFee = DecimalUtil.round(applyFee, 2);
		User u = userService.get(uid);
		double curFee = DecimalUtil.sub(DecimalUtil.sub(u.getTotal_fee(), u.getApply_fee()), u.getFinish_fee());
		if(curFee < applyFee) {
			return MJsonView.resp(BaseStatus.PARAM_INVALID, "账户余额不足");
		}
		int wid = userService.withdraw(uid, applyFee);
		if(wid > 0) {
			return MJsonView.ok();
		}
		return MJsonView.resp(BaseStatus.DB_ERROR);
	}
}
