package com.shanpin.sales.wap.module.user;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.shanpin.sales.wap.module.trade.TradeService;
import com.shanpin.sales.wap.module.trade.WithdrawService;
import com.shanpin.sales.wap.module.trade.model.Bonus;
import com.shanpin.sales.wap.module.trade.model.Trade;
import com.shanpin.sales.wap.module.user.model.User;
import com.shanpin.sales.wap.module.user.model.UserBind;
import com.shanpin.sales.wap.module.user.model.UserLogin;

@Service
public class UserService {

	@Autowired
	private TradeService tradeService;
	@Autowired
	private WithdrawService withdrawService;
	@Autowired
	private UserDao userDao;
	
	public User get(int uid) {
		return userDao.get(uid);
	}
	
	public User getByPhone(String phone) {
		return userDao.getByPhone(phone);
	}
	
	public User getByOpenId(String openId) {
		return userDao.getByOpenId(openId);
	}
	
	public UserLogin authAccount(String phone, String password) {
		UserLogin userLogin = new UserLogin();
		User u = userDao.verifyAccount(phone, password);
		if(u == null) {
			userLogin.setUid(0);
			userLogin.setPhone(null);
			userLogin.setOpenId(null);
			userLogin.setBind(UserBind.no);
		} else {
			userLogin.setUid(u.getUid());
			userLogin.setPhone(u.getPhone());
			userLogin.setOpenId(u.getOpen_id());
			userLogin.setBind(u.getBind());
		}
		return userLogin;
	}
	
	public UserLogin authOpenId(String openId) {
		UserLogin userLogin = new UserLogin();
		User u = userDao.getByOpenId(openId);
		if(u == null || u.getBind() == UserBind.no) {
			userLogin.setUid(0);
			userLogin.setPhone(null);
			userLogin.setOpenId(openId);
			userLogin.setBind(UserBind.no);
		} else {
			userLogin.setUid(u.getUid());
			userLogin.setPhone(u.getPhone());
			userLogin.setOpenId(openId);
			userLogin.setBind(u.getBind());
		}
		return userLogin;
	}
	
	/**
	 * 用户绑定号码
	 * @param phone
	 * @param password
	 * @param openId
	 * @return
	 */
	public int bind(String phone, String password, String openId) {
		User u = userDao.getByPhone(phone);
		//用户不存在
		if(u == null) {
			return userDao.add(phone, password, openId, UserBind.phone);
		}
		//用户存在(订单系统已推送数据过来)
		if (u.getBind() == UserBind.no) {
			int uid = u.getUid();
			boolean r = userDao.bind(uid, openId, password);
			if(r) {
				return uid;
			}
		}
		return 0;
	}
	
	/**
	 * 用户身份认证
	 * @param uid
	 * @param realName
	 * @param idCard
	 * @return
	 */
	public boolean identify(int uid, String realName, String idCard) {
		return userDao.identify(uid, realName, idCard);
	}

	public boolean findPassword(String phone, String password) {
		return userDao.findPassword(phone, password);
	}

	public Trade hasTrade(String phone) {
		return tradeService.hasTrade(phone);
	}

	public Bonus bonus(int uid) {
		return tradeService.bonus(uid);
	}

	public int withdraw(int uid, double applyFee) {
		boolean r = userDao.updateAccount(uid, applyFee);
		if(r) {
			int wid = withdrawService.add(uid, applyFee);
			return wid;
		}
		return 0;
	}
}
