package com.shanpin.sales.wap.module.webchat;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jetty.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.shanpin.sales.wap.module.webchat.model.Button;
import com.shanpin.sales.wap.module.webchat.model.Menu;
import com.shanpin.sales.wap.module.webchat.model.ViewButton;
import com.shanpin.sales.wap.utils.HttpsUtils;
import com.shanpin.sales.wap.webchat4j.common.Config;

/**
 * 微信公众号菜单构建工具类
 */
public class MenuBuider {
	
	private static Logger logger = LoggerFactory.getLogger(MenuBuider.class);
	
	/** 
	 * 服务号accessToken（GET） 
	 * 参数 appid appSecret
	 * */
	private final static String URL_ACCESS_TOKEN = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s";
	/** 
	 * 菜单创建（POST） 
	 * 参数accessToken
	 * */
	private final static String URL_MENU_CREATE = "https://api.weixin.qq.com/cgi-bin/menu/create?access_token=%s";
	/** 
	 * 菜单查询（GET） 
	 * 参数accessToken
	 * */
	private final static String URL_MENU_GET_URL = "https://api.weixin.qq.com/cgi-bin/menu/get?access_token=%s";
	/** 
	 * 菜单删除（GET）
	 * 参数accessToken
	 *  */
	private final static String URL_MENU_DELETE_URL = "https://api.weixin.qq.com/cgi-bin/menu/delete?access_token=%s";

	
	private static final String BUTTON_TYPE = "view";
	
	public static void main(String[] args) {

		ViewButton linggongziBtn = new ViewButton();
		linggongziBtn.setName("领工资");
		linggongziBtn.setType(BUTTON_TYPE);
		linggongziBtn.setUrl(OAuthService.getOauthUrl("http://wxsales.shanp.com/income/wap/user/account", "utf-8", OAuthService.SCOPE_SNSAPI_BASE));

		Menu menu = new Menu();
		menu.setButton(new Button[] {linggongziBtn});

		String accessToken = getAccessToken();
		boolean r = deleteMenu(accessToken);
		if(r) {
			r = createMenu(menu, accessToken);
		}
	}
	
	/**
	 * 获取服务号access_token
	 */
	public static String getAccessToken() {
		final String appid = Config.instance().getAppid();
		final String appSecret = Config.instance().getAppSecret();
		
		/** 获取access_token的接口地址（GET） 限2000（次/天）(服务号的token) */
		final String url = String.format(URL_ACCESS_TOKEN, appid, appSecret);
		logger.info("https access-token url:{}", url);
		
		String jsonStr = HttpsUtils.httpsRequest(url, HttpsUtils.METHOD_GET, null);
		if(StringUtil.isBlank(jsonStr)) {
			logger.info("https access-token error, empty result");
			return null;
		}
		String accessToken = null;
		ObjectMapper mapper = new ObjectMapper();
		JsonNode root = null;
		try {
			root = mapper.readTree(jsonStr);
			accessToken = root.path("access_token").asText();
		} catch (Exception e) {
			logger.error("https access-token error, result:{}", jsonStr, e);
		}
		logger.info("https access-token token:{}", accessToken);
		return accessToken;
	}
	
	/**
	 * 创建菜单
	 */
	public static boolean createMenu(Menu menu, String accessToken) {
		String url = String.format(URL_MENU_CREATE, accessToken);
		logger.info("https create-menu url:{}", url);
		String menuJsonStr = JSONObject.toJSONString(menu);
		String result = HttpsUtils.httpsRequest(url, HttpsUtils.METHOD_POST, menuJsonStr);
		boolean r = false;
		if(StringUtil.isNotBlank(result)) {
			ObjectMapper mapper = new ObjectMapper();
			try {
				JsonNode root = mapper.readTree(result);
				int errorCode = root.path("errcode").asInt();
				String errorMsg = root.path("errmsg").asText();
				if (errorCode == 0) {
					r = true;
				} else {
					logger.error("https create-menu error, errorCode:{} errorMsg:{}", errorCode, errorMsg);
					r = false;
				}
			} catch (Exception e) {
				logger.error("https create-menu error", e);
			}
		}
		logger.info("https create-menu result:{}", r);
		return r;
	}

	/**
	 * 查询菜单
	 */
	public static String getMenu(String accessToken) {
		String url = String.format(URL_MENU_GET_URL, accessToken);
		logger.info("https get-menu url:{}", url);
		String result = HttpsUtils.httpsRequest(url, HttpsUtils.METHOD_GET, null);
		logger.info("https get-menu result:{}", result);
		return result;
	}

	/**
	 * 删除菜单
	 */
	public static boolean deleteMenu(String accessToken) {
		String url = String.format(URL_MENU_DELETE_URL, accessToken);
		logger.info("https delete-menu url:{}", url);
		String result = HttpsUtils.httpsRequest(url, HttpsUtils.METHOD_GET, null);
		boolean r = false;
		if (StringUtils.isNotBlank(result)) {
			ObjectMapper mapper = new ObjectMapper();
			try {
				JsonNode root = mapper.readTree(result);
				int errorCode = root.path("errcode").asInt();
				String errorMsg = root.path("errmsg").asText();
				if (errorCode == 0) {
					r = true;
				} else {
					logger.error("https delete-menu error, errorCode:{} errorMsg:{}", errorCode, errorMsg);
					r = false;
				}
			} catch (Exception e) {
				logger.error("https delete-menu error", e);
			}
		}
		return r;
	}
}
