package com.shanpin.sales.wap.module.webchat;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jetty.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.shanpin.sales.wap.module.webchat.model.AccessTokenOAuth;
import com.shanpin.sales.wap.utils.HttpsUtils;
import com.shanpin.sales.wap.webchat4j.common.Config;

public class OAuthService {

	private static Logger logger = LoggerFactory.getLogger(OAuthService.class);

	//wechat oauth url
	public static String OAUTH = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=APPID&redirect_uri=REDIRECT_URI&response_type=code&scope=SCOPE&state=STATE#wechat_redirect";
	//通过oauth获取用户详细信息
	public static String GET_USER_INFO_OAUTH = "https://api.weixin.qq.com/sns/userinfo?access_token=ACCESS_TOKEN&openid=OPENID&lang=zh_CN";
	//获取oauth网页认证的token
	public static String GET_ACCESS_TOKEN_OAUTH = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=SECRET&code=CODE&grant_type=authorization_code";

	public static String SCOPE_SNSAPI_BASE = "snsapi_base";
	/**
	 * 获得Oauth认证的URL
	 * @param redirectUrl 跳转的url
	 * @param charset 字符集格式
	 * @param scope OAUTH scope
	 * @return oauth url
	 */
	public static String getOauthUrl(String redirectUrl, String charset, String scope) {
		String appid = Config.instance().getAppid();
		String url = OAUTH.replace("APPID", appid)
						.replace("REDIRECT_URI", urlEncode(redirectUrl, charset))
						.replace("SCOPE", scope);
		return url;
	}
	
	/**
	 * 获取Access_Token（oAuth认证
	 * 此access_token与基础支持的access_token不同）
	 * @param code 用户授权后得到的code
	 * @return AccessTokenOAuth对象
	 */
	public static AccessTokenOAuth getOAuthAccessToken(String code) {
		String appid = Config.instance().getAppid();
		String appSecret = Config.instance().getAppSecret();
		String url = GET_ACCESS_TOKEN_OAUTH
							.replace("APPID", appid)
							.replace("SECRET", appSecret)
							.replace("CODE", code);

		String jsonStr = HttpsUtils.httpsRequest(url, HttpsUtils.METHOD_POST, null);
		if(StringUtil.isBlank(jsonStr) ) {
			logger.error("OAuth access-token error, empty result");
			return null;
		}
		
		AccessTokenOAuth accessTokenOAuth = null;
		try {
			ObjectMapper mapper = new ObjectMapper();
			JsonNode root = mapper.readTree(jsonStr);
			if (StringUtils.isNotEmpty((String) root.path("errcode").asText())
					&& root.path("errcode").asInt() != 0) {
				logger.error("OAuth access-token error, errcode:{} errmsg:{}", 
						root.path("errcode").asInt(), root.path("errmsg").asText());
			} else {
				accessTokenOAuth = new AccessTokenOAuth();
				accessTokenOAuth.setAccessToken(root.path("access_token").asText());
				accessTokenOAuth.setExpiresIn(root.path("expires_in").asInt());
				accessTokenOAuth.setRefreshToken(root.path("refresh_token").asText());
				accessTokenOAuth.setOpenid(root.path("openid").asText());
				accessTokenOAuth.setScope(root.path("scope").asText());
			}
		} catch (IOException e) {
			logger.error("OAuth access-token error", e);
		}
		return accessTokenOAuth;
	}
	
	private static String urlEncode(String str, String charset) {
		String ret = "";
		try {
			ret = URLEncoder.encode(str, charset);
		} catch (UnsupportedEncodingException e) {
			logger.error("url encode error", e);
			ret = str;
		}
		return ret;
	}
}
