package com.shanpin.sales.wap.shiro;

import java.util.Set;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.subject.Subject;
import org.apache.shiro.util.CollectionUtils;
import org.apache.shiro.web.filter.authz.AuthorizationFilter;
import org.eclipse.jetty.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.shanpin.core.constant.EnvConf;
import com.shanpin.sales.wap.module.user.model.UserBind;
import com.shanpin.sales.wap.module.user.model.UserLogin;
import com.shanpin.sales.wap.module.webchat.OAuthService;
import com.shanpin.sales.wap.module.webchat.model.AccessTokenOAuth;

public class OpenIdAuthorizationFilter extends AuthorizationFilter {

	private static final Logger logger = LoggerFactory.getLogger(OpenIdAuthorizationFilter.class);

	@Override
	protected boolean isAccessAllowed(ServletRequest request,
			ServletResponse response, Object mappedValue) throws Exception {
		Subject subject = getSubject(request, response);
		String[] rolesArray = (String[]) mappedValue;
		if (rolesArray == null || rolesArray.length == 0) {
			return true;
		}
		Set<String> roles = CollectionUtils.asSet(rolesArray);
		for (String role : roles) {
			if (subject.hasRole(role)) {
				return true;
			}
		}
		return false;
	}

	@Override
	protected boolean onAccessDenied(ServletRequest request,
			ServletResponse response, Object mappedValue) throws Exception {
		String[] rolesArray = (String[]) mappedValue;
		if(rolesArray == null) {
			rolesArray = new String[0];
		}
		Set<String> roles = CollectionUtils.asSet(rolesArray);
		
		String code = request.getParameter("code");
		if(StringUtil.isNotBlank(code)) {
			String openId = null;
			if(EnvConf.isProd()) {
				try{
					AccessTokenOAuth token = OAuthService.getOAuthAccessToken(code);
					openId = token.getOpenid();
				} catch (Exception e) {
					logger.error("Wechat OAuthService-getOAuthAccessToken fail", e);
				}
			} else {
				openId = "123456";
			}
			if(StringUtil.isNotBlank(openId)) {
				Subject subject = SecurityUtils.getSubject();
				LoginToken token = new LoginToken(openId);
				try{
					subject.login(token);
				} catch (UnknownAccountException e) { //账号不存在
					request.getRequestDispatcher("/income/wap/user/bind").forward(request, response);
					return false;
				} catch (AuthenticationException e) { //认证失败
					request.getRequestDispatcher("/income/wap/user/bind").forward(request, response);
					return false;
				} catch (Exception e) { //未知异常
					request.getRequestDispatcher("/income/wap/user/bind").forward(request, response);
					return false;
				}
				UserLogin userLogin = (UserLogin)subject.getPrincipal();
				UserBind userBind = userLogin.getBind();
				if(userBind == UserBind.phone) {
					if(roles.contains("u")) {
						request.getRequestDispatcher("/income/wap/user/identify").forward(request, response);
						return false;
					}
					return true;
				} else if(userBind == UserBind.idCard) {
					request.getRequestDispatcher("/income/wap/user/account").forward(request, response);
					return false;
				}
				request.getRequestDispatcher("/income/wap/user/bind").forward(request, response);
				return false;
			}
		}
		if(roles.contains("n")) {
			return true;
		}
		saveRequestAndRedirectToLogin(request, response);
		return false;
	}
}
