package com.shanpin.sales.wap.shiro;

import javax.annotation.Resource;

import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authc.credential.AllowAllCredentialsMatcher;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.eclipse.jetty.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

import com.shanpin.sales.wap.module.user.UserService;
import com.shanpin.sales.wap.module.user.model.UserBind;
import com.shanpin.sales.wap.module.user.model.UserLogin;

public class ShiroRealm extends AuthorizingRealm {

	@Resource
	private UserService userService;

	@Autowired
	public ShiroRealm(ApplicationContext ctx) {
		super();
		setCredentialsMatcher(new AllowAllCredentialsMatcher());
		setAuthenticationTokenClass(LoginToken.class);
	}

	@Override
	protected AuthenticationInfo doGetAuthenticationInfo(
			AuthenticationToken authcToken) throws AuthenticationException {

		SimpleAuthenticationInfo info = null;
		LoginToken token = (LoginToken) authcToken;
		
		String phone = token.getPhone() == null ? null : token.getPhone().trim();
		String password = token.getPassword();
		String openId = token.getOpenId();
		if(StringUtil.isNotBlank(phone) && StringUtil.isNotBlank(password)) {
			//账号密码认证
			UserLogin userLogin = userService.authAccount(phone, password);
			info = new SimpleAuthenticationInfo(userLogin, password.toCharArray(), getName());
			return info;
		} else if(StringUtil.isNotBlank(openId)){
			//微信openId认证
			UserLogin userLogin = userService.authOpenId(openId);
			info = new SimpleAuthenticationInfo(userLogin, openId.toCharArray(), getName());
			return info;
		}
		return info;
	}

	@Override
	protected AuthorizationInfo doGetAuthorizationInfo(
			PrincipalCollection principals) {
		UserLogin userLogin = (UserLogin) principals.fromRealm(getName()).iterator().next();
		if (userLogin != null) {
			if(userLogin.getBind() == UserBind.phone) {
				SimpleAuthorizationInfo info = new SimpleAuthorizationInfo();
				info.addRole("p");
				return info;
			} else if(userLogin.getBind() == UserBind.idCard) {
				SimpleAuthorizationInfo info = new SimpleAuthorizationInfo();
				info.addRole("u");
				return info;
			}
		}
		return null;
	}
}
