package com.shanpin.sales.wap.utils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>https请求工具类
 */
public class HttpsUtils {

	private static Logger logger = LoggerFactory.getLogger(HttpsUtils.class);
	private static final String DEFAULT_CHARSET = "utf-8";
	
	public static final String METHOD_GET = "GET";
	public static final String METHOD_POST = "POST";
	
	/**
	 * 发送https请求
	 * @param url 请求地址
	 * @param method 请求方式（GET、POST）
	 * @param data 提交的数据
	 */
	public static String httpsRequest(String url, String method, String data) {
		String result = null;
		try {
			// 创建SSLContext对象，并使用我们指定的信任管理器初始化
			TrustManager[] tm = { new MyX509TrustManager() };
			SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
			sslContext.init(null, tm, new java.security.SecureRandom());
			// 从上述SSLContext对象中得到SSLSocketFactory对象
			SSLSocketFactory ssf = sslContext.getSocketFactory();

			HttpsURLConnection conn = (HttpsURLConnection) (new URL(url)).openConnection();
			conn.setSSLSocketFactory(ssf);

			conn.setDoOutput(true);
			conn.setDoInput(true);
			conn.setUseCaches(false);
			// 设置请求方式（GET/POST）
			conn.setRequestMethod(method);
			
			if (null != data) { // 当data不为null时向输出流写数据
				OutputStream outputStream = conn.getOutputStream();
				outputStream.write(data.getBytes(DEFAULT_CHARSET));
				outputStream.close();
			}

			// 从输入流读取返回内容
			BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream(), DEFAULT_CHARSET));
			String str = null;
			StringBuffer buf = new StringBuffer();
			while ((str = br.readLine()) != null) {
				buf.append(str);
			}
			br.close();
			conn.disconnect();
			result = buf.toString();
		} catch (Exception e) {
			logger.error("https request err", e);
		}
		return result;
	}
}

class MyX509TrustManager implements X509TrustManager {
	/** 检查客户端证书 */
	@Override
	public void checkClientTrusted(X509Certificate[] chain, String authType)
			throws CertificateException {

	}
	/** 检查服务器端证书 */
	@Override
	public void checkServerTrusted(X509Certificate[] chain, String authType)
			throws CertificateException {

	}
	/** 返回受信任的X509证书数组 */
	@Override
	public X509Certificate[] getAcceptedIssuers() {
		return null;
	}
}
