package com.shanpin.core.db;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * JDBC开发辅助类，指定数据库名称、表名称，生成从DB读取数据的文本代码
 * @author jahe.lai
 *
 */
public class GenMetaDataCode {
	
	private static Map<String, String> colMap = new HashMap<>();
	private static Map<String, String> methodMap = new HashMap<>();
	
	 static {
		 colMap.put("INT", "private int ");
		 colMap.put("BIT", "private int ");
		 colMap.put("VARCHAR", "private String ");
		 colMap.put("TEXT", "private String ");
		 colMap.put("DATE", "private Date ");
		 colMap.put("DATETIME", "private Date ");
		 colMap.put("TIMESTAMP", "private Date ");
		 colMap.put("DECIMAL", "private double ");

		 methodMap.put("INT", "rs.getInt");
		 methodMap.put("BIT", "rs.getInt");
		 methodMap.put("VARCHAR", "rs.getString");
		 methodMap.put("TEXT", "rs.getString");
		 methodMap.put("DATE", "rs.getDate");
		 methodMap.put("DATETIME", "rs.getTimestamp");
		 methodMap.put("TIMESTAMP", "rs.getTimestamp");
		 methodMap.put("DECIMAL", "rs.getDouble");
	}

	public static void main(String[] args) {
		getTableMetaData("shebao", "t_sb_city");
//		getTableMetaData("shebao_admin", "t_report_date_city");
	}
	

	private static void getTableMetaData(String databaseName, String tableName) {
		Connection conn = null;
		ResultSet rs = null;
		try {
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://192.168.2.199:3306/" + databaseName;
			String user = "root";
			String password = "shanpin";
			conn = DriverManager.getConnection(url, user, password);
			DatabaseMetaData dbMetaData = conn.getMetaData();

			genColumn(dbMetaData, tableName);
			genResultSetMethod(dbMetaData, tableName);
			
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			if (rs != null) {
				try {
					rs.close();
					
					if (null != conn) {
						conn.close();
					}
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	
	private static void genColumn(DatabaseMetaData dbMetaData, String tableName) throws SQLException {
		ResultSet rs = dbMetaData.getColumns(null, "%", tableName, "%");
		String colName = null, colTypeName = null;
		StringBuilder insertSqlBuilder = new StringBuilder("INSERT INTO " + tableName +" (");
		StringBuilder updateSqlBuilder = new StringBuilder("UPDATE " + tableName +" SET ");
		StringBuilder colBuilder = new StringBuilder(256);
		int columnCount = 0, colLen = 0;
		String newLine = "";
		while(rs.next()){
			columnCount++;
			newLine = columnCount%3==0 ? "\n" : "";
			colLen = colBuilder.length();
			colName = rs.getString("COLUMN_NAME");
			colTypeName = rs.getString("TYPE_NAME");
			
			insertSqlBuilder.append(rs.getString("COLUMN_NAME")+", "+newLine);
			updateSqlBuilder.append(rs.getString("COLUMN_NAME")+"=?, "+newLine);
			
			for(Map.Entry<String, String> entySet : colMap.entrySet()) {
				if (colTypeName.indexOf(entySet.getKey()) > -1) {
					colBuilder.append("/** " + rs.getString("REMARKS") + " */\n");
					colBuilder.append(entySet.getValue() + colName+";" + "\n");
					break;
				}
			}
			if (colBuilder.length() == colLen) {
				colBuilder.append("private String "+colName+";	  "  + colTypeName + "   // " + rs.getString("REMARKS") + "\n");
			}
	    }
		
		// print insertSqlBuilder
		insertSqlBuilder.deleteCharAt(insertSqlBuilder.length()-2).append(") VALUES (");
		for (int i = 1; i <= columnCount; i++) {
			insertSqlBuilder.append("?," + (i%3==0 ? " " : ""));
		}
		insertSqlBuilder.deleteCharAt(insertSqlBuilder.length()-1).append(")");
		System.out.println(insertSqlBuilder.toString());
		System.out.println("");
		
		// print insertSqlBuilder
		updateSqlBuilder.deleteCharAt(updateSqlBuilder.length()-2).append(" WHERE id=?");
		System.out.println(updateSqlBuilder.toString());
		System.out.println("");
		
		// print column
		System.out.println(colBuilder.toString());
		System.out.println("");
	}
	
	
	/**
	 * 输出从ResultSet获取数据的代码文本
	 * @param rsMetaData
	 * @throws SQLException
	 */
	private static void genResultSetMethod(DatabaseMetaData dbMetaData, String tableName) throws SQLException {
		ResultSet rs = dbMetaData.getColumns(null, "%", tableName, "%");
		int methodLen=0;
		String colName = null, colTypeName = null, text = null;
		StringBuilder methodBuilder = new StringBuilder(256);
		while(rs.next()){
			colName = rs.getString("COLUMN_NAME");
			colTypeName = rs.getString("TYPE_NAME");
			text = "pojo."+colName+" = ";
			
			methodLen = methodBuilder.length();
			for(Map.Entry<String, String> entySet : methodMap.entrySet()) {
				if (colTypeName.indexOf(entySet.getKey()) > -1) {
					methodBuilder.append(text + entySet.getValue()+"(\""+colName+"\");\n");
					break;
				}
			}
			if (methodBuilder.length() == methodLen) {
				methodBuilder.append(text + "rs.getString(\""+colName+"\");   " + colTypeName+"\n");
			}
		}
		methodBuilder.append("return pojo;");
		System.out.println(methodBuilder.toString());
	}

	
//	TABLE_CAT String => 表类别（可为null） 
//	TABLE_SCHEM String => 表模式（可为null） 
//	TABLE_NAME String => 表名称 
//	COLUMN_NAME String => 列名称 
//	DATA_TYPE int => 来自 java.sql.Types 的 SQL 类型 
//	TYPE_NAME String => 数据源依赖的类型名称，对于 UDT，该类型名称是完全限定的 
//	COLUMN_SIZE int => 列的大小。 
//	BUFFER_LENGTH 未被使用。 
//	DECIMAL_DIGITS int => 小数部分的位数。对于 DECIMAL_DIGITS 不适用的数据类型，则返回 Null。 
//	NUM_PREC_RADIX int => 基数（通常为 10 或 2） 
//	NULLABLE int => 是否允许使用 NULL。 
//		columnNoNulls - 可能不允许使用NULL值 
//		columnNullable - 明确允许使用NULL值 
//		columnNullableUnknown - 不知道是否可使用 null 
//	REMARKS String => 描述列的注释（可为null） 
//	COLUMN_DEF String => 该列的默认值，当值在单引号内时应被解释为一个字符串（可为null） 
//	SQL_DATA_TYPE int => 未使用 
//	SQL_DATETIME_SUB int => 未使用 
//	CHAR_OCTET_LENGTH int => 对于 char 类型，该长度是列中的最大字节数 
//	ORDINAL_POSITION int => 表中的列的索引（从 1 开始） 
//	IS_NULLABLE String => ISO 规则用于确定列是否包括 null。 
//		YES --- 如果参数可以包括 NULL 
//		NO --- 如果参数不可以包括 NULL 
//		空字符串 --- 如果不知道参数是否可以包括 null 
//	SCOPE_CATLOG String => 表的类别，它是引用属性的作用域（如果 DATA_TYPE 不是 REF，则为null） 
//	SCOPE_SCHEMA String => 表的模式，它是引用属性的作用域（如果 DATA_TYPE 不是 REF，则为null） 
//	SCOPE_TABLE String => 表名称，它是引用属性的作用域（如果 DATA_TYPE 不是 REF，则为null） 
//	SOURCE_DATA_TYPE short => 不同类型或用户生成 Ref 类型、来自 java.sql.Types 的 SQL 类型的源类型（如果 DATA_TYPE 不是 DISTINCT 或用户生成的 REF，则为null） 
//	IS_AUTOINCREMENT String => 指示此列是否自动增加 
//		YES --- 如果该列自动增加 
//		NO --- 如果该列不自动增加 
//		空字符串 --- 如果不能确定该列是否是自动增加参数  
}
