package com.shanpin.core.http;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.nio.charset.Charset;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class HttpClient {
	protected static Logger logger = LoggerFactory.getLogger(HttpClient.class);
	
	static CloseableHttpClient httpclients = HttpClients.createDefault();
	static RequestConfig requestConfig = RequestConfig.custom()  
			.setConnectionRequestTimeout(1000).setConnectTimeout(5000)  
	        .setSocketTimeout(5000).build();

	public static String post(String url, String content) throws IOException {
        return post(URI.create(url), new StringEntity(content));
	}
	
	public static String post(String url, InputStream input) throws IOException {
		return post(URI.create(url), new InputStreamEntity(input));
	}

	public static String post(URI url, HttpEntity entity) throws IOException {
        HttpPost post = new HttpPost(url);
        post.setConfig(requestConfig);
        // Content-Type必须指定
        post.setHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");
        post.setEntity(entity);
        try {
            HttpResponse httpResponse = httpclients.execute(post);
            if(httpResponse.getStatusLine().getStatusCode()!=200) {
                logger.warn("not 200: {}", httpResponse.getStatusLine());
            }
            return EntityUtils.toString(httpResponse.getEntity(), Charset.defaultCharset());
        } finally {
            post.releaseConnection();
        }
    }
	
	public static String get(String url) throws IOException {
		HttpGet get = new HttpGet(url);
		get.setConfig(requestConfig);
		get.setHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");
		try {
			HttpResponse httpResponse = httpclients.execute(get);
            if(httpResponse.getStatusLine().getStatusCode()!=200) {
                logger.warn("not 200: {}", httpResponse.getStatusLine());
            }
			return EntityUtils.toString(httpResponse.getEntity(), Charset.defaultCharset());
		} finally {
			get.releaseConnection();
		}
	}
	
//	public static void main(String[] args) throws IOException {
//		for (int i=0; i<1; i++) {
//			System.out.println(post("http://192.168.2.200:8000/sbl/base/list/city/v1", ""));
//			System.out.println(post("http://192.168.2.200:8000/sbl/user/login/v1", 
//					"phone=13631409050&password=827ccb0eea8a706c4c34a16891f84e7b"));			
//		}
//	}
	
}
