package com.shanpin.core.utils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *	校验工具类
 */
public class CheckUtils {
	
	public static boolean checkIdCard(String idCardStr) {
		if(idCardStr == null) {
			return false;
		}
		boolean result = false;
		String reg = "^[1-9]\\d{5}[1-9]\\d{3}((0[1-9])|(1[0-2]))((0[1-9])|([1-2]\\d)|3[0-1])((\\d{4})|\\d{3}[Xx])$";
		try {
			String idCard = idCardStr.trim();
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(idCard);
			boolean flag = matcher.matches();
			if (flag) {
				int[] factors = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
				int sum = 0;
				for (int i = 0; i < factors.length; i++) {
					int c = Integer.parseInt(String.valueOf(idCard.charAt(i)));
					sum += c * factors[i];
				}
				int remainder = sum % 11;
				/*获取身份证最后一位*/
				Character last = idCard.charAt(idCard.length() - 1);
				char[] lasts = {'1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2'};
				if (remainder == 2 && Character.toUpperCase(last) == 'X') {
					return true;
				}
				if (last == lasts[remainder]) {
					result = true;
				}
			}
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkEmail(String emailStr) {
		boolean result = false;
		String reg = "^\\w+((-\\w+)|(\\.\\w+))*\\@[A-Za-z0-9]+((\\.|-)[A-Za-z0-9]+)*\\.[A-Za-z0-9]+$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(emailStr.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkPhone(String phoneStr) {
		boolean result = false;
		String reg = "^0?(13|15|18|14|17)[0-9]{9}$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(phoneStr.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkTel(String telStr) {
		boolean result = false;
		String reg = "^[0-9\\-()（）]{7,18}$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(telStr.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkQQ(String qqStr) {
		boolean result = false;
		String reg = "^[1-9]*[1-9][0-9]*$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(qqStr.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkName(String nameStr) {
		boolean result = false;
		String reg = "^[A-Za-z\\u4e00-\\u9fa5]+$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(nameStr.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	/**
	 *	验证码 
	 */
	public static boolean checkVerifyCode(String verifyCode) {
		boolean result = false;
		String reg = "^[0-9]{4}$";
		try {
			Pattern regex = Pattern.compile(reg);
			Matcher matcher = regex.matcher(verifyCode.trim());
			result = matcher.matches();
		} catch (Exception e) {
			result = false;
		}
		return result;
	}
	
	public static boolean checkYYYYMM(String month) {
		if(month == null || month.trim().length() != 6) {
			return false;
		}
		boolean result = false;
		try{
			Integer.parseInt(month);
			result = true;
		} catch (Exception e) {
			result = false;
		}
		return result;
		
	}
}
