package com.shanpin.core.utils;

import java.sql.Timestamp;
import java.util.Date;

import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.PeriodType;

/**
 *	日期运算工具类 
 */
public class JodaUtil {
	
	public static java.sql.Timestamp now_ts() {
		return new Timestamp(DateTime.now().getMillis());
	}
	
	public static Date now() {
		return new Date(DateTime.now().getMillis());
	}
	
	public static Date parse(String dateStr) {
		return DateTime.parse(dateStr).toDate();
	}
	
	//加seconds秒
	public static Date plusSeconds(Date date, int seconds) {
		DateTime dateTime = new DateTime(date);
		return dateTime.plusSeconds(seconds).toDate();
	}
	
	//加minutes分
	public static Date plusMinutes(Date date, int minutes) {
		DateTime dateTime = new DateTime(date);
		return dateTime.plusMinutes(minutes).toDate();
	}
	
	//加day天
	public static Date plusDays(Date date, int days) {
		DateTime dateTime = new DateTime(date);
		return dateTime.plusDays(days).toDate();
	}
	
	//加months月
	public static Date plusMonths(Date date, int months) {
		DateTime dateTime = new DateTime(date);
		return dateTime.plusMonths(months).toDate();
	}
	
	//加years月
	public static Date plusYears(Date date, int years) {
		DateTime dateTime = new DateTime(date);
		return dateTime.plusYears(years).toDate();
	}
	
	/**
	 * 计算两个日期之间相差的天数
	 * @param beginDate 较小的时间
	 * @param endDate 较大的时间
	 * @return 返回相差天数，如果返回天数小于0，则返回0
	 */
	public static int daysCompare(Date beginDate,Date endDate){
		DateTime bDateTime = new DateTime(beginDate);
		DateTime eDateTime = new DateTime(endDate);
		Period p = new Period(bDateTime,eDateTime,PeriodType.days());
		return p.getDays();
	}
	
	public static int minutesCompare(Date beginDate,Date endDate){
		DateTime bDateTime = new DateTime(beginDate);
		DateTime eDateTime = new DateTime(endDate);
		Period p = new Period(bDateTime,eDateTime,PeriodType.minutes());
		return p.getMinutes();
	}
	
	public static Date getStart(Date date){
		DateTime dateTime = new DateTime(date);
		return dateTime.withTimeAtStartOfDay().toDate();
	}
	
	public static Date getEnd(Date date){
		DateTime dateTime = new DateTime(date);
		return dateTime.millisOfDay().withMaximumValue().toDate();
	}
	
	
	//获取一个月的第一天
	public static Date getMinDayOfMonth(Date date) {
		DateTime dateTime = new DateTime(date);
		return dateTime.dayOfMonth().withMinimumValue().toDate();
	}
	
	//获取一个月的最后一天
	public static Date getMaxDayOfMonth(Date date) {
		DateTime dateTime = new DateTime(date);
		return dateTime.dayOfMonth().withMaximumValue().toDate();
	}
	
	public static Date[] getAllDayOfMonth(Date date) {
		Date minDate = getMinDayOfMonth(date);
		Date maxDate = getMaxDayOfMonth(date);
		int days = daysCompare(minDate, maxDate);
		Date[] monthDates = new Date[days+1];
		for(int i=0; i<=days; i++) {
			monthDates[i] = plusDays(minDate, i);
		}
		return monthDates;
	}
	
	public static int dayOfWeek(Date date) {
		DateTime dateTime = new DateTime(date);
		return dateTime.dayOfWeek().get();
	}
}
