package com.shanpin.sales.logic.module.push;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.shanpin.core.http.HttpHelper;
import com.shanpin.core.utils.JsonUtil;
import com.shanpin.sales.logic.module.IUri;

@Controller
@RequestMapping("/wxsales")
public class PushController {

	private static Logger log = LoggerFactory.getLogger(PushController.class);
	
	@Autowired
	private PushService pushService;
	
	private static final String SUCCESS = "SUCCESS";
	static final List<String> WHITE_IPS = Collections.synchronizedList(new ArrayList<String>());
	
	static {
		WHITE_IPS.add("10.10.10.10");
		WHITE_IPS.add("127.0.0.1");
	}

	@RequestMapping(value=IUri.PUSH_ONE, method=RequestMethod.POST)
	public void getPush(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String remoteIp = HttpHelper.getIp(req);
		if (!WHITE_IPS.contains(remoteIp)) {
			HttpHelper.respText(resp, "IP-LIMIT.");
			return;
		}
		String params = HttpHelper.getParamFromStream(req);
		if (StringUtils.isBlank(params)) {
			HttpHelper.respText(resp, "REQUEST-BODY-BLANK.");
			return;
		}
		log.error("params={}", params);
		
		SalerOrder order = JsonUtil.toT(params, SalerOrder.class);
		String checkResult = order.check();
		if (StringUtils.isNoneBlank(order.check())) {
			HttpHelper.respText(resp, checkResult);
			return;
		}
		String mySign = SortSign.genSign(params, Key.SRV_ONE_KEY);
		if (!mySign.equals(order.getSign())) {
			HttpHelper.respText(resp, "SIGN-UNEQUAL");
			log.error("my-sign={}   req-sign={}", mySign, order.getSign());
			return;
		}
		boolean ret = pushService.save(order);
		HttpHelper.respText(resp, ret ? SUCCESS : "FAIL");
	}
	
}
