package com.shanpin.sales.logic.module.push;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.shanpin.sales.logic.db.AppDbRepository;
import com.shanpin.sales.logic.module.config.ConfigService;

@Repository
public class PushDao {
	
	private static Logger log = LoggerFactory.getLogger(PushDao.class);
	
	private static final CopyOnWriteArrayList<Integer> COM_HASHID_LIST = new CopyOnWriteArrayList<>();

	private static final CopyOnWriteArrayList<String> PHONE_LIST = new CopyOnWriteArrayList<>();
	
	@Autowired
	private AppDbRepository jdbc;
	
	@PostConstruct
	private void loadCompanyHashIds() {
		List<Integer> list = jdbc.queryForList("SELECT hashId FROM t_company", Integer.class);
		COM_HASHID_LIST.addAll(list);
		log.warn("INITED Company-hashId-List, size={}", list.size());
		
		List<String> phoneList = jdbc.queryForList("SELECT phone FROM t_user", String.class);
		PHONE_LIST.addAll(phoneList);
		log.warn("INITED Phone-List, size={}", phoneList.size());
	}
	

	@Transactional
	public boolean save(SalerOrder order, int hashId) {
		saveCompany(order, hashId);
		float income = ConfigService.calculateIncome(order.getTotal_fee());
		String sql = "INSERT INTO t_user_trade (trade_no, phone, total_fee,"
				+ " trade_time, com_hashId, income,"
				+ " gen_time) VALUES (?,?,?, ?,?,?, now())";
		boolean ret = jdbc.update(sql, order.getTrade_no(), order.getPhone(), order.getTotal_fee(),
				order.getTrade_time(), hashId, income);
		if (ret) {
			saveUser(order.getPhone(), income);
		}
		return ret;
	}
	
	/**
	 * 数据库增加一条公司记录
	 * @param order
	 * @param hashId
	 */
	private void saveCompany(SalerOrder order, int hashId) {
		if (COM_HASHID_LIST.contains(hashId)) {
			return;
		}
		COM_HASHID_LIST.add(hashId);
		String sql = "INSERT INTO t_company (hashId, company, companyId,"
				+ " gen_time) VALUES (?,?,?, now())";
		boolean ret = jdbc.update(sql, hashId, order.getCompany(), order.getCompany_id());
		
		if (!ret) {
			log.error("COM INSERT error. {}, {}, {}", 
					hashId, order.getCompany(), order.getCompany_id());
		}
	}
	private void saveUser(String phone, float income) {
		if (PHONE_LIST.contains(phone)) {
			updateTotal_fee(phone, income);
			return;
		}
		PHONE_LIST.add(phone);
		String sql = "INSERT INTO t_user (phone, total_fee) VALUES (?,?)";
		boolean ret = jdbc.update(sql, phone, income);
		
		if (!ret) {
			log.error("User INSERT error. {}", phone);
		}
	}
	
	private void updateTotal_fee(String phone, float total_fee) {
		String sql = "UPDATE t_user SET total_fee=total_fee+? WHERE phone=?";
		boolean ret = jdbc.update(sql, total_fee, phone);
		if (ret) {
			log.warn("update total_fee {} {}", phone, total_fee);
			return;
		}
		log.error("update total_fee error. {} {}", phone, total_fee);
	}
	
}
